@@ -9,12 +9,23 @@ from django.db.models import Prefetch
 
 from modelcluster.models import get_all_child_relations
 from modelcluster.queryset import FakeQuerySet
+from modelcluster.utils import ManyToManyTraversalError
 
-from tests.models import Band, BandMember, Place, Restaurant, SeafoodRestaurant, Review, Album, \
-    Article, Author, Category, Person, Room, House, Log, Dish, MenuItem, Wine
+from tests.models import Band, BandMember, Chef, Feature, Place, Restaurant, SeafoodRestaurant, \
+    Review, Album, Article, Author, Category, Person, Room, House, Log, Dish, MenuItem, Wine
 
 
 class ClusterTest(TestCase):
+
+    @classmethod
+    def setUpClass(cls) -> None:
+        super().setUpClass()
+        cls.gordon_ramsay = Chef.objects.create(name="Gordon Ramsay")
+        cls.strawberry_fields = Restaurant.objects.create(name="Strawberry Fields", proprietor=cls.gordon_ramsay)
+
+        cls.marco_pierre_white = Chef.objects.create(name="Marco Pierre White")
+        cls.the_yellow_submarine = Restaurant.objects.create(name="The Yellow Submarine", proprietor=cls.marco_pierre_white)
+
     def test_can_create_cluster(self):
         beatles = Band(name='The Beatles')
 
@@ -96,11 +107,6 @@ class ClusterTest(TestCase):
         self.assertRaises(BandMember.DoesNotExist, lambda: beatles.members.get(name='Reginald Dwight'))
         self.assertRaises(BandMember.MultipleObjectsReturned, lambda: beatles.members.get())
 
-        self.assertEqual([('Paul McCartney',)], beatles.members.filter(name='Paul McCartney').values_list('name'))
-        self.assertEqual(['Paul McCartney'], beatles.members.filter(name='Paul McCartney').values_list('name', flat=True))
-        # quick-and-dirty check that we can invoke values_list with empty args list
-        beatles.members.filter(name='Paul McCartney').values_list()
-
         self.assertTrue(beatles.members.filter(name='Paul McCartney').exists())
         self.assertFalse(beatles.members.filter(name='Reginald Dwight').exists())
 
@@ -136,6 +142,129 @@ class ClusterTest(TestCase):
         # queries on beatles.members should now revert to SQL
         self.assertTrue(beatles.members.extra(where=["tests_bandmember.name='John Lennon'"]).exists())
 
+    def test_values_list(self):
+        beatles = Band(
+            name="The Beatles",
+            members=[
+                BandMember(name="John Lennon", favourite_restaurant=self.strawberry_fields),
+                BandMember(name="Paul McCartney", favourite_restaurant=self.the_yellow_submarine),
+                BandMember(name="George Harrison"),
+                BandMember(name="Ringo Starr"),
+            ],
+        )
+
+        # Not specifying 'fields' should return a tuple of all field values
+        self.assertEqual(
+            [
+                # ID, band_id, name, favourite_restaurant_id
+                (None, None, 'Paul McCartney', self.the_yellow_submarine.id)
+            ],
+            list(beatles.members.filter(name='Paul McCartney').values_list())
+        )
+
+        NAME_ONLY_TUPLE = ('Paul McCartney',)
+
+        # Specifying 'fields' should return a tuple of just those field values
+        self.assertEqual([NAME_ONLY_TUPLE], list(beatles.members.filter(name='Paul McCartney').values_list('name')))
+
+        # 'fields' can span relationships using '__'
+        members = beatles.members.all().values_list('name', 'favourite_restaurant__proprietor__name')
+        self.assertEqual(
+            list(members),
+            [
+                ("John Lennon", "Gordon Ramsay"),
+                ("Paul McCartney", "Marco Pierre White"),
+                ("George Harrison", None),
+                ("Ringo Starr", None),
+            ]
+        )
+
+        # Ordering on the related fields will work too, and items with `None`` values will appear first
+        self.assertEqual(
+            list(members.order_by('favourite_restaurant__proprietor__name')),
+            [
+                ("George Harrison", None),
+                ("Ringo Starr", None),
+                ("John Lennon", "Gordon Ramsay"),
+                ("Paul McCartney", "Marco Pierre White"),
+
+            ]
+        )
+
+        # get() should return a tuple if used after values_list()
+        self.assertEqual(NAME_ONLY_TUPLE, beatles.members.filter(name='Paul McCartney').values_list('name').get())
+
+        # first() should return a tuple if used after values_list()
+        self.assertEqual(NAME_ONLY_TUPLE, beatles.members.filter(name='Paul McCartney').values_list('name').first())
+
+        # last() should return a tuple if used after values_list()
+        self.assertEqual(NAME_ONLY_TUPLE, beatles.members.filter(name='Paul McCartney').values_list('name').last())
+
+        # And the 'flat' argument should work as it does in Django
+        self.assertEqual(['Paul McCartney'], list(beatles.members.filter(name='Paul McCartney').values_list('name', flat=True)))
+
+        # Filtering or ordering after using values_list() should not raise an error
+        beatles.members.values_list("name").filter(name__contains="n").order_by("name")
+
+    def test_values(self):
+        beatles = Band(
+            name="The Beatles",
+            members=[
+                BandMember(name="John Lennon", favourite_restaurant=self.strawberry_fields),
+                BandMember(name="Paul McCartney", favourite_restaurant=self.the_yellow_submarine),
+                BandMember(name="George Harrison"),
+                BandMember(name="Ringo Starr"),
+            ],
+        )
+
+        # Not specifying 'fields' should return dictionaries with all field values
+        self.assertEqual(
+            [
+                {"id": None, "band": None, "name": "Paul McCartney", "favourite_restaurant": self.the_yellow_submarine.id}
+            ],
+            list(beatles.members.filter(name='Paul McCartney').values())
+        )
+
+        NAME_ONLY_DICT = {"name": "Paul McCartney"}
+
+        # Specifying 'fields' should return a dictionary of just those field values
+        self.assertEqual([NAME_ONLY_DICT], list(beatles.members.filter(name='Paul McCartney').values('name')))
+
+        # 'fields' can span relationships using '__'
+        members = beatles.members.all().values('name', 'favourite_restaurant__proprietor__name')
+        self.assertEqual(
+            list(members),
+            [
+                {"name": "John Lennon", "favourite_restaurant__proprietor__name": "Gordon Ramsay"},
+                {"name": "Paul McCartney", "favourite_restaurant__proprietor__name": "Marco Pierre White"},
+                {"name": "George Harrison", "favourite_restaurant__proprietor__name": None},
+                {"name": "Ringo Starr", "favourite_restaurant__proprietor__name": None},
+            ]
+        )
+
+        # Ordering on the related fields will work too, and items with `None`` values will appear first
+        self.assertEqual(
+            list(members.order_by('favourite_restaurant__proprietor__name')),
+            [
+                {"name": "George Harrison", "favourite_restaurant__proprietor__name": None},
+                {"name": "Ringo Starr", "favourite_restaurant__proprietor__name": None},
+                {"name": "John Lennon", "favourite_restaurant__proprietor__name": "Gordon Ramsay"},
+                {"name": "Paul McCartney", "favourite_restaurant__proprietor__name": "Marco Pierre White"},
+            ]
+        )
+
+        # get() should return a dict if used after values()
+        self.assertEqual(NAME_ONLY_DICT, beatles.members.filter(name='Paul McCartney').values('name').get())
+
+        # first() should return a dict if used after values_list()
+        self.assertEqual(NAME_ONLY_DICT, beatles.members.filter(name='Paul McCartney').values('name').first())
+
+        # last() should return a dict if used after values_list()
+        self.assertEqual(NAME_ONLY_DICT, beatles.members.filter(name='Paul McCartney').values('name').last())
+
+        # Filtering or ordering after using values() should not raise an error
+        beatles.members.values("name").filter(name__contains="n").order_by("name")
+
     def test_related_manager_assignment_ops(self):
         beatles = Band(name='The Beatles')
         john = BandMember(name='John Lennon')
@@ -265,10 +394,7 @@ class ClusterTest(TestCase):
         self.assertEqual(3, beatles.members.filter(band=also_beatles).count())
 
     def test_queryset_filtering_on_models_with_inheritance(self):
-        strawberry_fields = Restaurant.objects.create(name='Strawberry Fields')
-        the_yellow_submarine = SeafoodRestaurant.objects.create(name='The Yellow Submarine')
-
-        john = BandMember(name='John Lennon', favourite_restaurant=strawberry_fields)
+        john = BandMember(name='John Lennon', favourite_restaurant=self.strawberry_fields)
         ringo = BandMember(name='Ringo Starr', favourite_restaurant=Restaurant.objects.get(name='The Yellow Submarine'))
 
         beatles = Band(name='The Beatles', members=[john, ringo])
@@ -281,7 +407,7 @@ class ClusterTest(TestCase):
 
         # queried instance is more specific
         self.assertEqual(
-            list(beatles.members.filter(favourite_restaurant=the_yellow_submarine)),
+            list(beatles.members.filter(favourite_restaurant=self.the_yellow_submarine)),
             [ringo]
         )
 
@@ -478,6 +604,147 @@ class ClusterTest(TestCase):
             "one person died"
         )
 
+    def test_queryset_filtering_accross_foreignkeys(self):
+        band = Band(
+            name="The Beatles",
+            members=[
+                BandMember(name="John Lennon", favourite_restaurant=self.strawberry_fields),
+                BandMember(name="Ringo Starr", favourite_restaurant=self.the_yellow_submarine)
+            ],
+        )
+
+        # Filter over a single relationship
+        # ---------------------------------------
+        # Using the default/exact lookup type
+        self.assertEqual(
+            tuple(band.members.filter(favourite_restaurant__name="Strawberry Fields")),
+            (band.members.get(name="John Lennon"),)
+        )
+        self.assertEqual(
+            tuple(band.members.filter(favourite_restaurant__name="The Yellow Submarine")),
+            (band.members.get(name="Ringo Starr"),)
+        )
+        # Using an alternative lookup type
+        self.assertEqual(
+            tuple(band.members.filter(favourite_restaurant__name__icontains="straw")),
+            (band.members.get(name="John Lennon"),)
+        )
+        self.assertEqual(
+            tuple(band.members.filter(favourite_restaurant__name__icontains="yello")),
+            (band.members.get(name="Ringo Starr"),)
+        )
+
+        # Filtering over 2 relationships
+        # ---------------------------------------
+        # Using a default/exact field lookup
+        self.assertEqual(
+            tuple(band.members.filter(favourite_restaurant__proprietor__name="Gordon Ramsay")),
+            (band.members.get(name="John Lennon"),)
+        )
+        self.assertEqual(
+            tuple(band.members.filter(favourite_restaurant__proprietor__name="Marco Pierre White")),
+            (band.members.get(name="Ringo Starr"),)
+        )
+        # Using an alternative lookup type
+        self.assertEqual(
+            tuple(band.members.filter(favourite_restaurant__proprietor__name__iexact="gORDON rAMSAY")),
+            (band.members.get(name="John Lennon"),)
+        )
+        self.assertEqual(
+            tuple(band.members.filter(favourite_restaurant__proprietor__name__iexact="mARCO pIERRE wHITE")),
+            (band.members.get(name="Ringo Starr"),)
+        )
+        # Using an exact proprietor comparisson
+        self.assertEqual(
+            tuple(band.members.filter(favourite_restaurant__proprietor=self.gordon_ramsay)),
+            (band.members.get(name="John Lennon"),)
+        )
+        self.assertEqual(
+            tuple(band.members.filter(favourite_restaurant__proprietor=self.marco_pierre_white)),
+            (band.members.get(name="Ringo Starr"),)
+        )
+
+    def test_filtering_via_reverse_foreignkey(self):
+        band = Band(
+            name="The Beatles",
+            members=[
+                BandMember(name="John Lennon"),
+                BandMember(name="Ringo Starr"),
+            ],
+        )
+        self.assertEqual(
+            tuple(band.members.filter(band__name="The Beatles")),
+            tuple(band.members.all())
+        )
+        self.assertEqual(
+            tuple(band.members.filter(band__name="The Monkeys")),
+            ()
+        )
+
+    def test_ordering_accross_foreignkeys(self):
+        band = Band(
+            name="The Beatles",
+            members=[
+                BandMember(name="John Lennon", favourite_restaurant=self.strawberry_fields),
+                BandMember(name="Ringo Starr", favourite_restaurant=self.the_yellow_submarine),
+            ],
+        )
+
+        # Ordering accross a single relationship
+        # ---------------------------------------
+        self.assertEqual(
+            tuple(band.members.order_by("favourite_restaurant__name")),
+            (
+                band.members.get(name="John Lennon"),
+                band.members.get(name="Ringo Starr"),
+            )
+        )
+        # How about ordering in reverse?
+        self.assertEqual(
+            tuple(band.members.order_by("-favourite_restaurant__name")),
+            (
+                band.members.get(name="Ringo Starr"),
+                band.members.get(name="John Lennon"),
+            )
+        )
+
+        # Ordering accross 2 relationships
+        # --------------------------------
+        self.assertEqual(
+            tuple(band.members.order_by("favourite_restaurant__proprietor__name")),
+            (
+                band.members.get(name="John Lennon"),
+                band.members.get(name="Ringo Starr"),
+            )
+        )
+        # How about ordering in reverse?
+        self.assertEqual(
+            tuple(band.members.order_by("-favourite_restaurant__proprietor__name")),
+            (
+                band.members.get(name="Ringo Starr"),
+                band.members.get(name="John Lennon"),
+            )
+        )
+
+    def test_filtering_via_manytomany_raises_exception(self):
+        bay_window = Feature.objects.create(name="Bay window", desirability=6)
+        underfloor_heating = Feature.objects.create(name="Underfloor heading", desirability=10)
+        open_fire = Feature.objects.create(name="Open fire", desirability=3)
+        log_burner = Feature.objects.create(name="Log burner", desirability=10)
+
+        modern_living_room = Room.objects.create(name="Modern living room", features=[bay_window, underfloor_heating, log_burner])
+        classic_living_room = Room.objects.create(name="Classic living room", features=[bay_window, open_fire])
+
+        modern_house = House.objects.create(name="Modern house", address="1 Yellow Brick Road", main_room=modern_living_room)
+        classic_house = House.objects.create(name="Classic house", address="3 Yellow Brick Road", main_room=classic_living_room)
+
+        tenant = Person(
+            name="Alex", houses=[modern_house, classic_house]
+        )
+
+        with self.assertRaises(ManyToManyTraversalError):
+            tenant.houses.filter(main_room__features__name="Bay window")
+
     def test_prefetch_related(self):
         Band.objects.create(name='The Beatles', members=[
             BandMember(id=1, name='John Lennon'),
