@@ -9,16 +9,33 @@ from tests.models import Band, BandMember, Album, Restaurant, Article, Author, D
 from modelcluster.forms import ClusterForm
 from django.forms import Textarea, CharField
 from django.forms.widgets import TextInput, FileInput
+from django.utils.safestring import SafeString
 
 import datetime
 
 
 class ClusterFormTest(TestCase):
+    def test_cluster_form_with_no_formsets(self):
+        class BandForm(ClusterForm):
+            class Meta:
+                model = Band
+                fields = ['name']
+
+        self.assertFalse(BandForm.formsets)
+
+        beatles = Band(name='The Beatles')
+        form = BandForm(instance=beatles)
+        form_html = form.as_p()
+        self.assertIsInstance(form_html, SafeString)
+        self.assertInHTML('<label for="id_name">Name:</label>', form_html)
+        self.assertInHTML('<label for="id_albums-0-name">Name:</label>', form_html, count=0)
+
     def test_cluster_form(self):
         class BandForm(ClusterForm):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets = ['members', 'albums']
 
         self.assertTrue(BandForm.formsets)
 
@@ -30,13 +47,17 @@ class ClusterFormTest(TestCase):
         form = BandForm(instance=beatles)
 
         self.assertEqual(5, len(form.formsets['members'].forms))
-        self.assertTrue('albums' in form.as_p())
+        form_html = form.as_p()
+        self.assertIsInstance(form_html, SafeString)
+        self.assertInHTML('<label for="id_name">Name:</label>', form_html)
+        self.assertInHTML('<label for="id_albums-0-name">Name:</label>', form_html)
 
     def test_empty_cluster_form(self):
         class BandForm(ClusterForm):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets = ['members', 'albums']
 
         form = BandForm()
 
@@ -47,6 +68,7 @@ class ClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets = ['members', 'albums']
 
         beatles = Band(name='The Beatles', members=[
             BandMember(name='George Harrison'),
@@ -130,6 +152,7 @@ class ClusterFormTest(TestCase):
                     }
                 }
                 fields = ['name']
+                formsets = ['members', 'albums']
 
         form = BandForm()
         self.assertEqual(Textarea, type(form['name'].field.widget))
@@ -155,6 +178,46 @@ class ClusterFormTest(TestCase):
         self.assertNotIn('release_date', form.formsets['albums'].forms[0].fields)
         self.assertEqual(Textarea, type(form.formsets['albums'].forms[0]['name'].field.widget))
 
+    def test_without_kwarg_inheritance(self):
+        # by default, kwargs passed to the ClusterForm do not propagate to child forms
+        class BandForm(ClusterForm):
+            class Meta:
+                model = Band
+                formsets = {
+                    'members': {'fields': ['name']}
+                }
+                fields = ['name']
+
+        form = BandForm(label_suffix="!!!:")
+        form_html = form.as_p()
+        # band name field should have label_suffix applied
+        self.assertInHTML('<label for="id_name">Name!!!:</label>', form_html)
+        # but this should not propagate to member form fields
+        self.assertInHTML('<label for="id_members-0-name">Name!!!:</label>', form_html, count=0)
+
+    def test_with_kwarg_inheritance(self):
+        # inherit_kwargs should allow kwargs passed to the ClusterForm to propagate to child forms
+        class BandForm(ClusterForm):
+            class Meta:
+                model = Band
+                formsets = {
+                    'members': {'fields': ['name'], 'inherit_kwargs': ['label_suffix']}
+                }
+                fields = ['name']
+
+        form = BandForm(label_suffix="!!!:")
+        form_html = form.as_p()
+        # band name field should have label_suffix applied
+        self.assertInHTML('<label for="id_name">Name!!!:</label>', form_html)
+        # and this should propagate to member form fields too
+        self.assertInHTML('<label for="id_members-0-name">Name!!!:</label>', form_html)
+
+        # the form should still work without a label_suffix kwarg
+        form = BandForm()
+        form_html = form.as_p()
+        self.assertInHTML('<label for="id_name">Name:</label>', form_html)
+        self.assertInHTML('<label for="id_members-0-name">Name:</label>', form_html)
+
     def test_custom_formset_form(self):
         class AlbumForm(ClusterForm):
             pass
@@ -207,6 +270,7 @@ class ClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets = ['members', 'albums']
 
         form = BandFormWithFFC()
         self.assertEqual(Textarea, type(form['name'].field.widget))
@@ -217,6 +281,7 @@ class ClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets = ['members', 'albums']
 
         john = BandMember(name='John Lennon')
         paul = BandMember(name='Paul McCartney')
@@ -257,48 +322,6 @@ class ClusterFormTest(TestCase):
         self.assertTrue(BandMember.objects.filter(name='George Harrison').exists())
         self.assertFalse(BandMember.objects.filter(name='John Lennon').exists())
 
-    def test_can_omit_formset_from_submission(self):
-        """
-        If no explicit `formsets` parameter has been given, any formsets missing from the
-        submission should be skipped over.
-        https://github.com/wagtail/wagtail/issues/5414#issuecomment-567468127
-        """
-        class BandForm(ClusterForm):
-            class Meta:
-                model = Band
-                fields = ['name']
-
-        john = BandMember(name='John Lennon')
-        paul = BandMember(name='Paul McCartney')
-        abbey_road = Album(name='Abbey Road')
-        beatles = Band(name='The Beatles', members=[john, paul], albums=[abbey_road])
-        beatles.save()
-
-        form = BandForm({
-            'name': "The Beatles",
-
-            'members-TOTAL_FORMS': 3,
-            'members-INITIAL_FORMS': 2,
-            'members-MAX_NUM_FORMS': 1000,
-
-            'members-0-name': john.name,
-            'members-0-DELETE': 'members-0-DELETE',
-            'members-0-id': john.id,
-
-            'members-1-name': paul.name,
-            'members-1-id': paul.id,
-
-            'members-2-name': 'George Harrison',
-            'members-2-id': '',
-        }, instance=beatles)
-        self.assertTrue(form.is_valid())
-        form.save()
-
-        beatles = Band.objects.get(id=beatles.id)
-        self.assertEqual(1, beatles.albums.count())
-        self.assertTrue(BandMember.objects.filter(name='George Harrison').exists())
-        self.assertFalse(BandMember.objects.filter(name='John Lennon').exists())
-
     def test_cannot_omit_explicit_formset_from_submission(self):
         """
         If an explicit `formsets` parameter has been given, formsets missing from a form submission
@@ -346,6 +369,7 @@ class ClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets = ['members', 'albums']
 
         john = BandMember(name='John Lennon')
         paul = BandMember(name='Paul McCartney')
@@ -392,6 +416,7 @@ class ClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets = ['members', 'albums']
 
         form = BandForm({
             'name': "The Beatles",
@@ -432,6 +457,7 @@ class ClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets = ['members', 'albums']
 
         form = BandForm()
         form_html = form.as_p()
@@ -443,6 +469,7 @@ class ClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets = ['members', 'albums']
 
         form = BandForm({
             'name': "The Beatles",
@@ -482,6 +509,7 @@ class ClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets = ['members', 'albums']
 
         please_please_me = Album(name='Please Please Me', release_date=datetime.date(1963, 3, 22))
         beatles = Band(name='The Beatles', albums=[please_please_me])
@@ -539,7 +567,7 @@ class ClusterFormTest(TestCase):
         beatles.save()
         self.assertEqual(0, Band.objects.get(id=beatles.id).albums.count())
 
-    def test_cluster_form_without_formsets(self):
+    def test_cluster_form_with_empty_formsets_list(self):
         class BandForm(ClusterForm):
             class Meta:
                 model = Band
@@ -566,6 +594,7 @@ class ClusterFormTest(TestCase):
             class Meta:
                 model = Restaurant
                 fields = ['name', 'tags', 'serves_hot_dogs', 'proprietor']
+                formsets = ['menu_items', 'reviews', 'tagged_items']
 
         self.assertIn('reviews', RestaurantForm.formsets)
 
@@ -635,6 +664,7 @@ class ClusterFormTest(TestCase):
             class Meta:
                 model = Restaurant
                 fields = ['name', 'tags', 'serves_hot_dogs', 'proprietor']
+                formsets = ['menu_items', 'reviews', 'tagged_items']
                 widgets = {
                     'name': WidgetWithMedia
                 }
@@ -717,6 +747,7 @@ class ClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets = ['members', 'albums']
 
         form = BandForm({
             'name': "The Beatles",
@@ -818,11 +849,37 @@ class FormWithM2MTest(TestCase):
 
 class NestedClusterFormTest(TestCase):
 
+    def test_no_nested_formsets_without_explicit_formset_definition(self):
+        class BandForm(ClusterForm):
+            class Meta:
+                model = Band
+                fields = ['name']
+                formsets=['members', 'albums']
+
+        self.assertTrue(BandForm.formsets)
+
+        beatles = Band(name='The Beatles', albums=[
+            Album(name='Please Please Me', songs=[
+                Song(name='I Saw Her Standing There'),
+                Song(name='Misery')
+            ]),
+        ])
+
+        form = BandForm(instance=beatles)
+
+        self.assertEqual(4, len(form.formsets['albums'].forms))
+        self.assertNotIn('songs', form.formsets['albums'].forms[0].formsets)
+        self.assertNotIn('songs', form.as_p())
+
     def test_nested_formsets(self):
         class BandForm(ClusterForm):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets={
+                    'members': [],
+                    'albums': {'formsets': ['songs']}
+                }
 
         self.assertTrue(BandForm.formsets)
 
@@ -844,6 +901,10 @@ class NestedClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets={
+                    'members': [],
+                    'albums': {'formsets': ['songs']}
+                }
 
         form = BandForm()
         self.assertEqual(3, len(form.formsets['albums'].forms))
@@ -854,6 +915,10 @@ class NestedClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets={
+                    'members': [],
+                    'albums': {'formsets': ['songs']}
+                }
 
         beatles = Band(name='The Beatles', albums=[
             Album(name='Please Please Me', songs=[
@@ -908,7 +973,6 @@ class NestedClusterFormTest(TestCase):
         self.assertTrue(Song.objects.filter(name='Misery').exists())
         self.assertFalse(Song.objects.filter(name='I Saw Her Standing There').exists())
 
-    @unittest.skip('Explicit nested formsets not yet enabled')
     def test_explicit_nested_formset_list(self):
         class BandForm(ClusterForm):
             class Meta:
@@ -925,7 +989,6 @@ class NestedClusterFormTest(TestCase):
         self.assertTrue('albums' in form.as_p())
         self.assertTrue('songs' in form.as_p())
 
-    @unittest.skip('Excluded nested formsets not yet enabled')
     def test_excluded_nested_formset_list(self):
         class BandForm(ClusterForm):
             class Meta:
@@ -946,6 +1009,10 @@ class NestedClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets={
+                    'members': [],
+                    'albums': {'formsets': ['songs']}
+                }
 
         first_song = Song(name='I Saw Her Standing There')
         second_song = Song(name='Misery')
@@ -1001,6 +1068,10 @@ class NestedClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets={
+                    'members': [],
+                    'albums': {'formsets': ['songs']}
+                }
 
         first_song = Song(name='I Saw Her Standing There')
         second_song = Song(name='Misery')
@@ -1054,6 +1125,10 @@ class NestedClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets={
+                    'members': [],
+                    'albums': {'formsets': ['songs']}
+                }
 
         form = BandForm({
             'name': "The Beatles",
@@ -1103,6 +1178,10 @@ class NestedClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets={
+                    'members': [],
+                    'albums': {'formsets': ['songs']}
+                }
 
         form = BandForm()
         form_html = form.as_p()
@@ -1114,6 +1193,10 @@ class NestedClusterFormTest(TestCase):
             class Meta:
                 model = Band
                 fields = ['name']
+                formsets={
+                    'members': [],
+                    'albums': {'formsets': ['songs']}
+                }
 
         form = BandForm({
             'name': "The Beatles",
